/*
 * 2012-3 Red Hat Inc. and/or its affiliates and other contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.overlord.rtgov.activity.processor.mvel;

import org.mvel2.MVEL;
import org.overlord.rtgov.activity.model.ActivityType;
import org.overlord.rtgov.activity.processor.InformationTransformer;

/**
 * This class represents a MVEL based information transformer.
 *
 */
public class MVELInformationTransformer extends InformationTransformer {

    private String _expression=null;
    
    private Object _compiledExpression=null;
    
    /**
     * This method returns the expression.
     * 
     * @return The expression
     */
    public String getExpression() {
        return (_expression);
    }
    
    /**
     * This method sets the expression.
     * 
     * @param expr The expression
     */
    public void setExpression(String expr) {
        _expression = expr;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void init() throws Exception {
        super.init();
        
        // Initialize the mvel script expression
        _compiledExpression = MVEL.compileExpression(getExpression());
    }
    
    /**
     * {@inheritDoc}
     */
    public String transform(Object information, java.util.Map<String, Object> headers, ActivityType activityType) {
        java.util.Map<String,Object> vars=new java.util.HashMap<String, Object>();
        vars.put("information", information);
        vars.put("headers", headers);
        vars.put("activity", activityType);
        
        Object result=MVEL.executeExpression(_compiledExpression, vars);  
        
        return (result == null ? null : result.toString());
    }
    
}
